#include "environ.h"
#include "gtypes.h"
#include <SDL/SDL.h>

Uint8 cursor_definition[] = {
/* MSB first!  */	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,	
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 
};	

Uint8 cursor_data[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

/* Configuration options */
const char* const  short_options = "hi:dz:flr:"; 
const struct option long_options[] = {
/* { "long option name", next argument, NULL, 'short option' } */
	{ "help",       0, NULL, 'h' },
	{ "iterations", 1, NULL, 'i' },
	{ "zoom",       1, NULL, 'z' },
	{ "fullscreen", 0, NULL, 'f' },
	{ "driver",     0, NULL, 'd' },
	{ "listmodes",  0, NULL, 'l' },
	{ "resolution", 1, NULL, 'r' },
	{ NULL,         0, NULL,  0  } /* This line required at the end of array */
};


/* -------------------------------------------------- */
/* Debug function that prints SDL_Surface information */
/* -------------------------------------------------- */

void PrintSurfaceInfo(SDL_Surface *surface){
	printf("Surface flags: 0x%x\n", surface->flags);
	printf("Pixel format:\n\tBits per pixel: %d\n\tBytes per pixel: %d\n",                  
		 surface->format->BitsPerPixel,
         surface->format->BytesPerPixel);
	printf("Width and height of the surface: %d x %d\n", surface->w, surface->h);                
	printf("Length of a surface scanline in bytes: %d Bytes\n", surface->pitch);              
	printf("Pointer to the actual pixel data: %p\n", surface->pixels);
}

/* --------------------------------------------------- */
/* Get list of available modes with fullscreen support */
/* --------------------------------------------------- */
char **GetAvailableModes(void){
/* Use this function after SDL_Init() !!!! */
	int i, j = 0;
	char previous[10]; /* Buffer for screen resolutions */
	char mode[10];
	char **list;
	SDL_Rect **gfx_modes = SDL_ListModes(NULL, SDL_FULLSCREEN|SDL_SWSURFACE);

	list = (char **) malloc(20 * sizeof(char *));
	for(i = 0; i < 20; i++){
		list[i] = malloc(sizeof(mode) * sizeof(char *));	
	}

	for(i = 0; gfx_modes[i]; i++){
	/* ANSI C does not define: */
	/* snprintf(mode, sizeof(mode), "%dx%d", gfx_modes[i]->w, gfx_modes[i]->h); 	*/

		sprintf(mode, "%dx%d", gfx_modes[i]->w, gfx_modes[i]->h);	
		if(strncmp(previous, mode, sizeof(mode)!=0)){
			strncpy(list[j], mode, sizeof(mode));		
			j++;
			if(j>18)
				break;
		}
		strncpy(previous, mode, sizeof(mode));
	}
	list[j] = '\0';
	return list;
}

/* ------------------------- */
/* Print graphics modes list */
/* ------------------------- */
void PrintModeList(FILE *stream, char **available)
{
/* address of 'available' modes returned by GetAvailableModes() */
	int j = 0;
	while(available[j]){
		fprintf(stream, "\t%s\n", available[j]);
		j++;
	}
}

/* --------------------- */
/* Set screen resolution */
/* --------------------- */
void SetScreenResolution(char *mode, char **available){
/* address of 'available' modes returned by GetAvailableModes() */
	Uint16 width, height, i = 0;
	SDL_bool set = SDL_FALSE;
	char *number;
  	
	number = strchr(mode, 'x');

	if(number){ /* 'x' pressented */
		while(available[i]){
			if(!strcmp(mode, available[i])){
				set = SDL_TRUE;
				break;
			}
			++i;
		}

		if(set){
			height = atoi(number+1);
			*number = '\0';
			width = atoi(mode);
			*number = 'x';
		}
		else {
			fprintf(stderr,"Unsuported graphics mode '%s'!\n", mode);
			SDL_Quit();
			exit(1);
		}
	}
	else {
		fprintf(stderr,"Bad argument -r %s!\nGet list of supported resolutions with -l parameter.\n", mode);
		SDL_Quit();
		exit(1);
	}
	
	MAIN_OPTIONS.Screen_height = height;
	MAIN_OPTIONS.Screen_width = width;
}

/* -------------------- */
/* Print help in shell  */
/* -------------------- */
void PrintHelp(FILE* stream, int exitcode){
	fprintf(stream,
			"  -h  --help         Show this usage informations.\n"
			"  -i  --iterations   Set number of computing iterations.\n"
			"  -z  --zoom         Set step of zooming.\n"
			"  -f  --fullscreen   Run in fullscreen.\n"
			"  -d  --driver       Display graphics driver.\n"
			"  -l  --listmodes    Display available display resolutions.\n"
			"  -r  --resolution   Set screen resolution (see -r for available modes).\n"
			"\n"
		);
	exit(exitcode);
}

/* --------------------------- */
/* Command line configuration  */
/* --------------------------- */
void Configuration(int argc, char *argv[]){
	int next_option;
	MAIN_OPTIONS.MaxIter = DEFAULT_MAXITER;
	MAIN_OPTIONS.Zoom_step = DEFAULT_ZOOM_STEP;
	MAIN_OPTIONS.Screen_width = DEFAULT_WIDTH;
	MAIN_OPTIONS.Screen_height = DEFAULT_HEIGHT;
	
	do {
		next_option = getopt_long (argc, argv, short_options, long_options, NULL);
		
		switch (next_option)
		{
			case 'h': /* -h or --help */
					fprintf(stdout, "Usage: %s [options]\n", argv[0]);
					PrintHelp(stdout, 0);
				break;
			case 'i':  /* -i or --iterations */
					MAIN_OPTIONS.MaxIter = atoi(optarg);			
				break;
			case 'z':  /* -z or --zoom */
					MAIN_OPTIONS.Zoom_step = atof(optarg);			
				break;
			case 'f':  /* -f or --fullscreen */
					MAIN_OPTIONS.Fullscreen = SDL_TRUE;			
				break;
			case 'd': /* -d or --driver */
					MAIN_OPTIONS.Display_driver = SDL_TRUE;
				break;
			case 'l': /* -l or --listmodes */
					MAIN_OPTIONS.List_modes = SDL_TRUE;
				break;
			case 'r': /* -r or --resolution */
					strncpy(MAIN_OPTIONS.Mode_param, optarg, sizeof(MAIN_OPTIONS.Mode_param));
				break;
			default:
				break;
		}
	} while (next_option != -1);
	
}

/* -------------------------------- */
/* Program initialization functions */
/* -------------------------------- */
void Init(void){
	Uint32 flags = FLAGS;

	if(MAIN_OPTIONS.Fullscreen == SDL_TRUE)
			flags = flags|SDL_FULLSCREEN;
	
	if(SDL_Init(SDL_INIT_VIDEO) < 0){
     fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
     exit(1);
   }
	
	avail_gfx_modes = GetAvailableModes();	

	/* Print available graphics modes list and quit */
	if(MAIN_OPTIONS.List_modes == SDL_TRUE){
		fprintf(stdout, "Available graphics modes:\n");
		PrintModeList(stdout, avail_gfx_modes);
		SDL_Quit();
		exit(0);
	}

	/* Show graphics driver and quit */
	if(MAIN_OPTIONS.Display_driver == SDL_TRUE){
		char driver[16];
		SDL_VideoDriverName(driver, 16);
		printf("Used graphics driver: %s\n", driver);
		SDL_Quit();
		exit(0);
	}

	/* Change resolution */
	if(strlen(MAIN_OPTIONS.Mode_param)>0){
		SetScreenResolution(MAIN_OPTIONS.Mode_param, avail_gfx_modes); 
		printf("Set resolution: %d x %d\n", MAIN_OPTIONS.Screen_width, MAIN_OPTIONS.Screen_height);
		MAIN_OPTIONS.Mandel_pos.x = MAIN_OPTIONS.Screen_width / 2;
	}
	
	/* Program window */
	Main_win = SDL_SetVideoMode(MAIN_OPTIONS.Screen_width, MAIN_OPTIONS.Screen_height, BPP, flags);	

	/* Memory space for sets computations */
	Mandelbrot = SDL_CreateRGBSurface(FLAGS, MAIN_OPTIONS.Screen_width / 2, MAIN_OPTIONS.Screen_height, BPP, 
		COLOR_BIT_MASK);
	Julia = SDL_CreateRGBSurface(FLAGS, MAIN_OPTIONS.Screen_width / 2, MAIN_OPTIONS.Screen_height, BPP, 
	  COLOR_BIT_MASK);
	
	if((cursor_mask = CursorMask(cursor_definition, 16, 16))==NULL){
		fprintf(stderr, "User cursor not defined!\n");
	}
	/* Switch my cursor on */
	cursor = SDL_CreateCursor(cursor_data, cursor_mask, 16, 16, 7, 7);
	SDL_SetCursor(cursor);
}

/* -------------- */
/* Do before exit */
/* -------------- */
void Deinit(void){
	SDL_FreeSurface(Mandelbrot);
	SDL_FreeSurface(Main_win);
	SDL_FreeCursor(cursor);
	SDL_Quit();
	free(cursor_mask);
	free(avail_gfx_modes);
}


Uint8 *CursorMask(Uint8 *bitmap, Uint8 x_size, Uint8 y_size){
/*
   Uint8 *bitmap -- bitmap with bit cursor definiton
   Uint8 x_size 
   Uint8 y_size
   return addres of cursor mask data
*/
	int i;
	Uint8 out = 0, *mask;
	int weight = 128;
	
	if(!((x_size * y_size) % 8)){
		mask = (Uint8 *) malloc( (x_size*y_size*sizeof(Uint8)) / 8 );

		for(i = 0; i < x_size*y_size; i++)
		{
			out +=  bitmap[i] * (Uint8) weight;
			weight = weight >> 1;
			if(!weight)
			{
				*(mask+i/8) = out;
				weight = 128;
				out = 0;
			}
		}
		return mask;
	}
	else
		return NULL;
}

