/* Definitions for types and global variables */

#ifndef _GTYPES_H
#define _GTYPES_H

#include <SDL/SDL.h>

#define PROGRAM_NAME "Small Fractal Explorer v. 0.2alpha"
#define BPP	    32
#define DEFAULT_WIDTH  800
#define DEFAULT_HEIGHT 600
#define DEFAULT_MANDEL_POS DEFAULT_WIDTH/2
#define FLAGS  SDL_SWSURFACE

#define OUTPUT_FILE       "sfe_test.bmp"
#define DEFAULT_MAXITER   150
#define DEFAULT_ZOOM_STEP 1.25
#define DEFAULT_COLOR_MAP PalColorGreen 

/* number of palette functions from gfx.c */
#define NUMBER_COLOR_MAP  7

#define ZOOM_RECT_COLOR 0xFF, 0xFF, 0xFF, 128

/* Order of bytes in real heterogenous word */
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	#define	COLOR_BIT_MASK 0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000
#else
	#define	COLOR_BIT_MASK 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff 
#endif

/* Surfaces for image data storaga */
SDL_Surface *Main_win;
SDL_Surface *Mandelbrot;
SDL_Surface *Julia;

/* Mask of my cursor definition */
Uint8 *cursor_mask;	

/* Address of SDL cursor definiton data */
SDL_Cursor *cursor;

/* List of available resolutions, we get them in Init() */
char **avail_gfx_modes;

/* Complex Number */
typedef struct {
	double real;
	double imag;
} cnum;

/* Dimensions of real plane */ 
typedef struct  {
	double XMin;
	double XMax;
	double YMin;
	double YMax;
} RealPlane;

/* Pointer to function which map number of iterations to color */
typedef void (*T_ColorMap)(SDL_Surface *, unsigned int, unsigned int, unsigned int);

extern T_ColorMap Array_ColorMap[];

/* Main Option Structure */
typedef struct {
	unsigned short int MaxIter; /* Maximal iteration number for computing sets */

	/* Default values */
	RealPlane Default_Mandel_plane;  
	RealPlane Default_Julia_plane;
	cnum Default_Start_seed;
	SDL_Rect Mandel_pos;
	SDL_Rect Julia_pos;
	
	/* User values */
	RealPlane Mandel_plane;
	RealPlane Julia_plane;
	cnum Start_seed;
	SDL_bool Redraw_Julia;
	SDL_bool Zoom_Mandel;
	SDL_bool Zoom_Julia;
	double Zoom_step;
	
	/* Program environment */
	SDL_bool Fullscreen;
	SDL_bool Display_driver;
	SDL_bool List_modes;
	SDL_bool Screen_width;
	SDL_bool Screen_height;
	char 	 Mode_param[10];
	/* Pointer to function that convert number of iteration to RGBA */
	T_ColorMap  ColorMap;
} Options;

/* In MAIN_OPTIONS are stored all important data 
 for program environment. */
Options MAIN_OPTIONS;

#endif
