/*
	+--------------------------------------+ 
 	|      Small Fractal Explorer          |
	|                                      |
	|  Autor : Martin Bruchanov            |
	|  www   : http://bruxy.regnet.cz/sfe  |	 
	|  email : bruxy at regnet dot cz      |
	|                                      |
	|                            (c) 2005  |
	+--------------------------------------+ 

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* System libraries */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>


/* #include "cursor.h" */
#include "gtypes.h"
#include "gfx.h"
#include "environ.h"

/******************************
 *                            *
 *     Global vars  values,   *
 *   declaration in gtypes.h  *
 *                            *
 *****************************/

unsigned int mouse_x, mouse_y;
unsigned int ColorMap_index = 0;


Options MAIN_OPTIONS = {
	0,                        /* Maximal iteration number for computing sets */
	/* Initial values */
	{ -2.1, 1.0, -2.2, 2.2 }, /* Default_Mandel_plane; */ 
	{ -2.0, 2.0, -2.5, 2.5 }, /* Default_Julia_plane; */
	{  0.0, 0.0},             /* Default_Start_seed */
	{ DEFAULT_MANDEL_POS, 0, 0, 0,},  /* Position of Mandelbrot plane in main window */
	{  0, 0, 0, 0,},          /* Position of Julia plane in main window */
	
	/* Program variables  */ 
	{ -2.1, 1.0, -2.2, 2.2 }, /* Mandel_plane */ 
	{ -2.0, 2.0, -2.5, 2.5 }, /* Julia_plane */
	{  0.0, 0.0},             /* Start_seed */
	SDL_TRUE,                 /* Redraw_Julia */
	SDL_TRUE,                 /* Zoom Mandelbrot plane */
	SDL_FALSE,	              /* Zoom Julia plane */
	0,	                      /* Zoom step */
	/* Program environment */
	SDL_FALSE,                /* Fullscreen Mode */
	SDL_FALSE,                /* Display screen driver */
	SDL_FALSE,                /* List modes */
	0,                        /* Screen width */
	0,                        /* Screen height */
	"",                       /* Mode parameter from shell */
	NULL                      /* Pointer to function */
};


/********************
 *                  *
 *    Functions     *
 *                  *
 ********************/

/* --------------------------- */
/* Read real function position */
/* --------------------------- */

void ReadPosition(int mouse_x, int mouse_y ,int width, int height, 
		RealPlane *LeftPlane, RealPlane *RightPlane, cnum *coord)
{
	/* Main window is divided into two parts left and right, so we need
	   read diferent coordinates from both */
	const int half_width = width / 2; 
	const double LDeltaX = (LeftPlane->XMax - LeftPlane->XMin) / (double) half_width;
	const double LDeltaY = (LeftPlane->YMax - LeftPlane->YMin) / (double) height;
	const double RDeltaX = (RightPlane->XMax - RightPlane->XMin) / (double) half_width;
	const double RDeltaY = (RightPlane->YMax - RightPlane->YMin) / (double) height;
	double c_real, c_imag;
	
	/* Reading from Julia -- LeftPlane */
	if(mouse_x < half_width){
		c_imag = LeftPlane->YMin + LDeltaY * mouse_y;
		c_real = LeftPlane->XMin + LDeltaX * mouse_x;
	}
	else /* Reading from MandelBrot */
	{
		c_imag = RightPlane->YMin + RDeltaY * mouse_y;
		c_real = RightPlane->XMin + RDeltaX * ( mouse_x - half_width );
	}

	if(coord){
		coord->imag = c_imag;
		coord->real = c_real;
	}
}

/* ---------------- */
/* Keyboard control */
/* ---------------- */

int KeyboardHandler(SDL_keysym key){
	switch(key.sym){
		case 'f': case 'F': /* Fullscreen toggle on/off */
			/* This function is in SDL implented only under LINUX */
			MAIN_OPTIONS.Fullscreen = SDL_TRUE;
			if(!SDL_WM_ToggleFullScreen(Main_win)){
				fprintf(stderr, "Error when switching to fullscreen!\n");
				MAIN_OPTIONS.Fullscreen = SDL_FALSE;
			}
			break;
		case 'c': case 'C': /* Cycle Palete */
				ColorMap_index++;
				if(ColorMap_index >= NUMBER_COLOR_MAP)
				{
					ColorMap_index = 0;
				}

				MAIN_OPTIONS.ColorMap = Array_ColorMap[ColorMap_index];
			
				MAIN_OPTIONS.Redraw_Julia = SDL_TRUE;
				MAIN_OPTIONS.Zoom_Mandel = SDL_TRUE;
			break;
		case 's': case 'S':
			if(SDL_SaveBMP(Main_win, OUTPUT_FILE)==-1){
				fprintf(stderr, "Error saving file!\n");
			}
			break;
		case 'r': case 'R': /* Reset to default state */
			MAIN_OPTIONS.Mandel_plane = MAIN_OPTIONS.Default_Mandel_plane;
			MAIN_OPTIONS.Julia_plane  = MAIN_OPTIONS.Default_Julia_plane;
			MAIN_OPTIONS.Start_seed   = MAIN_OPTIONS.Default_Start_seed;
			MAIN_OPTIONS.Zoom_Julia   = SDL_TRUE;
			MAIN_OPTIONS.Redraw_Julia = SDL_TRUE;
			MAIN_OPTIONS.Zoom_Mandel  = SDL_TRUE;
			break;
		case SDLK_ESCAPE: case 'q': case 'Q': /* QUIT */
			return SDL_FALSE;
			break;
		case 'h': case 'H': /* Display Help */
			break;
		default:
			break;
	}
	return SDL_TRUE;
}

/*----------------------*/
/* Mouse button control */
/*----------------------*/

int MouseHandler(SDL_Event event){
	/* TODO:
	LeftButton reads coordinates.
	RightButton zooming.
	Wheel Up zoom in.
	Wheel Down zoom out.
   */
	static SDL_bool right_state;
	static int x_click;
 	static int y_click;

	double XSize, YSize;
	cnum center;
	
	if((event.type ==  SDL_MOUSEBUTTONDOWN)){
		/* Mouse wheel zooming  ********************************************* */
		ReadPosition(event.button.x, event.button.y, Main_win->w, Main_win->h, 
			&MAIN_OPTIONS.Julia_plane, &MAIN_OPTIONS.Mandel_plane, &center);
		switch(event.button.button){
			case SDL_BUTTON_LEFT: /* Single click for start seed reading */
					if(event.button.x > MAIN_OPTIONS.Screen_width/2){ /* from Mandelbrot plane */
					 	ReadPosition(event.button.x, event.button.y, Main_win->w, Main_win->h,
								&MAIN_OPTIONS.Julia_plane, &MAIN_OPTIONS.Mandel_plane, &MAIN_OPTIONS.Start_seed);
						MAIN_OPTIONS.Redraw_Julia = SDL_TRUE;
						return 0;
					}
						else return 0;
				break;
			case SDL_BUTTON_WHEELUP: /* Zoom in */
				if(event.button.x > MAIN_OPTIONS.Screen_width/2){ /* Zoom in Mandelbrot */
					XSize = ((MAIN_OPTIONS.Mandel_plane.XMax - MAIN_OPTIONS.Mandel_plane.XMin) / MAIN_OPTIONS.Zoom_step)/2.0;
					YSize = ((MAIN_OPTIONS.Mandel_plane.YMax - MAIN_OPTIONS.Mandel_plane.YMin) / MAIN_OPTIONS.Zoom_step)/2.0;
					MAIN_OPTIONS.Zoom_Mandel = SDL_TRUE;
				/*	printf("Wheel up = %f x %f\n", center.real, center.imag); */
				}
				else{ /* Zoom in Julia */
					XSize = ((MAIN_OPTIONS.Julia_plane.XMax - MAIN_OPTIONS.Julia_plane.XMin) / MAIN_OPTIONS.Zoom_step)/2.0;
					YSize = ((MAIN_OPTIONS.Julia_plane.YMax - MAIN_OPTIONS.Julia_plane.YMin) / MAIN_OPTIONS.Zoom_step)/2.0;
					MAIN_OPTIONS.Zoom_Julia = SDL_TRUE;
				}
				break;
			case SDL_BUTTON_WHEELDOWN: /* Zoom out */
				if(event.button.x > MAIN_OPTIONS.Screen_width/2){ /* Zoom out Mandelbrot */
					XSize = ((MAIN_OPTIONS.Mandel_plane.XMax - MAIN_OPTIONS.Mandel_plane.XMin) * MAIN_OPTIONS.Zoom_step)/2.0;
					YSize = ((MAIN_OPTIONS.Mandel_plane.YMax - MAIN_OPTIONS.Mandel_plane.YMin) * MAIN_OPTIONS.Zoom_step)/2.0;
					MAIN_OPTIONS.Zoom_Mandel = SDL_TRUE;
				/* printf("Wheel down = %f x %f\n", center.real, center.imag); */
				}	
				else{ 
					XSize = ((MAIN_OPTIONS.Julia_plane.XMax - MAIN_OPTIONS.Julia_plane.XMin) * MAIN_OPTIONS.Zoom_step)/2.0;
					YSize = ((MAIN_OPTIONS.Julia_plane.YMax - MAIN_OPTIONS.Julia_plane.YMin) * MAIN_OPTIONS.Zoom_step)/2.0;
					MAIN_OPTIONS.Zoom_Julia = SDL_TRUE;
				}
	
				break;
			default:
				break;
		}
		if(MAIN_OPTIONS.Zoom_Mandel == SDL_TRUE){	
				MAIN_OPTIONS.Mandel_plane.XMin = center.real - XSize;
				MAIN_OPTIONS.Mandel_plane.XMax = center.real + XSize;
				MAIN_OPTIONS.Mandel_plane.YMin = center.imag - YSize;
				MAIN_OPTIONS.Mandel_plane.YMax = center.imag + YSize;
		} 
		if(MAIN_OPTIONS.Zoom_Julia == SDL_TRUE){
				MAIN_OPTIONS.Julia_plane.XMin = center.real - XSize;
				MAIN_OPTIONS.Julia_plane.XMax = center.real + XSize;
				MAIN_OPTIONS.Julia_plane.YMin = center.imag - YSize;
				MAIN_OPTIONS.Julia_plane.YMax = center.imag + YSize;
		}
		return 0;
	}
	/* ****************************************************************** */

	
	/* Mouse event */
/*	if((event.type == SDL_MOUSEMOTION) || (event.type == SDL_MOUSEBUTTONDOWN) || (event.type ==  SDL_MOUSEBUTTONUP)) */
	if(event.type == SDL_MOUSEMOTION)
	{
		if(SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(SDL_BUTTON_LEFT)){
		/* Read coordinates */
		 	ReadPosition(event.motion.x, event.motion.y, Main_win->w, Main_win->h, &MAIN_OPTIONS.Julia_plane, &MAIN_OPTIONS.Mandel_plane, &MAIN_OPTIONS.Start_seed);
			if(event.motion.x > MAIN_OPTIONS.Screen_width/2)
				MAIN_OPTIONS.Redraw_Julia = SDL_TRUE;
			/*
			printf("Left pressed - %d x %d\n", event.motion.x, event.motion.y); 
			printf("Readed seed = %f x %f\n", MAIN_OPTIONS.Start_seed.real,  MAIN_OPTIONS.Start_seed.imag);
			*/
		}
	
#if 0
		/* Select zoom area. */
		if(SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(SDL_BUTTON_RIGHT)){
			/* Remember coordinates value */
			if(event.button.state == SDL_PRESSED){
				printf("Right pressed - %d x %d\n", event.button.x, event.button.y);
				right_state = SDL_TRUE;
				x_click = event.button.x;
				y_click = event.button.y;	
			}
			printf("Right pressed motion - %d x %d -> %d x %d\n", x_click, y_click, event.motion.x, event.motion.y);	
		
			SDL_BlitSurface(Mandelbrot, NULL, Main_win, NULL);
			SDL_UpdateRect(Main_win, 0, 0, 0, 0);
			DrawZoomArea(Main_win,  x_click, y_click , event.motion.x, event.motion.y);
			return 0;	
		}
		if(event.button.state == SDL_RELEASED && right_state == SDL_TRUE){
			printf("Right released - %d x %d\n", event.button.x, event.button.y);
			SDL_BlitSurface(Mandelbrot, NULL, Main_win, NULL);
			SDL_UpdateRect(Main_win, 0, 0, 0, 0);
			right_state = SDL_FALSE;
		}
		/* **************** */
#endif
	}
	return 0;
}

/* ------------- */
/* Event handler */
/* ------------- */
SDL_bool ProcessEvent(void)
{
	SDL_Event event;

	while(SDL_PollEvent(&event))
	{
		MouseHandler(event);

		switch(event.type)
		{
			/* Keyboard events */
			case SDL_KEYDOWN:
				if(KeyboardHandler(event.key.keysym) == SDL_FALSE){
					return SDL_FALSE;
				}
				break;
			/* Quit from WM */
			case SDL_QUIT:
					return SDL_FALSE;
				break;

			default:
				break;
		}
	}
	return SDL_TRUE;
}

/********************
 *                  *
 *       Main       *
 *                  *
 ********************/

int main(int argc, char *argv[]){
	SDL_bool quit = SDL_FALSE;
	/* Uint32 i = 0; */

	/* Configuration from command line */
	Configuration(argc, argv);
	
	/* Initialization */	
	Init();

	MAIN_OPTIONS.ColorMap = DEFAULT_COLOR_MAP;
	
	/* Main program loop ************************************************* */
	while(!quit){
		/* Redraw Julia set with new random seed or zoom Julia plane */
		if(MAIN_OPTIONS.Redraw_Julia == SDL_TRUE || MAIN_OPTIONS.Zoom_Julia == SDL_TRUE){
			JuliaSet(Julia, Julia->w, Julia->h, MAIN_OPTIONS.Julia_plane, 
				MAIN_OPTIONS.Start_seed, MAIN_OPTIONS.MaxIter, MAIN_OPTIONS.ColorMap);
			SDL_BlitSurface(Julia, NULL, Main_win, &MAIN_OPTIONS.Julia_pos);
				}
		/* Zoom Mandelbrot plane */
		if(MAIN_OPTIONS.Zoom_Mandel == SDL_TRUE){
			MandelbrotSet(Mandelbrot, Mandelbrot->w, Mandelbrot->h, 
				MAIN_OPTIONS.Mandel_plane, MAIN_OPTIONS.MaxIter, MAIN_OPTIONS.ColorMap);
			SDL_BlitSurface(Mandelbrot, NULL, Main_win, &MAIN_OPTIONS.Mandel_pos);
		}
		
		/* Redraw main window */
		if(MAIN_OPTIONS.Redraw_Julia == SDL_TRUE || 
			MAIN_OPTIONS.Zoom_Julia == SDL_TRUE ||
			MAIN_OPTIONS.Zoom_Mandel == SDL_TRUE){
				SDL_UpdateRect(Main_win, 0, 0, 0, 0);
				MAIN_OPTIONS.Redraw_Julia = SDL_FALSE;
				MAIN_OPTIONS.Zoom_Julia = SDL_FALSE;
				MAIN_OPTIONS.Zoom_Mandel = SDL_FALSE;
		}
				
		quit = !ProcessEvent();
	}
	/* ***************** ************************************************* */

	/* Exit */
	Deinit();
	exit(0);
}


